"""Provides python bindings to LuauAST and LuauRenderer."""
# LuauAST: /robloxpyc/LuauAST (Typescript)
# LuauRenderer: /robloxpyc/LuauRenderer (Typescript)
# Script Path /robloxpyc/bindings.py

# @AsynchronousAI - 2023
import subprocess

class TypescriptBridge:
    def __init__(path):
        pass
    #I have no idea how to do this, but I'll try to figure it out. Maybe use C and make a bridge, or use JSON files to communicate and subprocess. (Most likely).